import Flutter
import UIKit

public class SwiftBundleRavePayPlugin: NSObject, FlutterPlugin {
    var bundlePayFlutterResult: FlutterResult!
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "bundle_rave_pay", binaryMessenger: registrar.messenger())
    let instance = SwiftBundleRavePayPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    
    if call.method == "initializePayment" {
        self.bundlePayFlutterResult = result
        processPayment(call)
    }
  }
    
    private func processPayment(_ call: FlutterMethodCall) {
        guard let dictionary = (call.arguments as! [String: Any])["body"] as? [String: Any] else { return }
        guard let publicKey = dictionary["publicKey"],
        let secretKey = dictionary["secretKey"],
        let currency = dictionary["currencyCode"],
        let transactionRef = dictionary["transactionReference"],
        let email = dictionary["email"],
        let isCard = dictionary["isCard"],
        let isLive = dictionary["isLive"] else {
            bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Missing some required data", details: nil))
            return
        }
        
        let raveManager: RavePayManager!
        
        if ("\(isCard)" == "false") {
            raveManager = RavePayManager()
            raveManager.supportedPaymentMethods = [.account]
        } else {
            guard let cardExpiry = dictionary["cardExpiry"],
            let cardNumber = dictionary["cardNumber"],
            let cvv = dictionary["cvv"] else {
                bundlePayFlutterResult(FlutterError(code: "INVALID DATA", message: "Missing some required data", details: nil))
                return
            }
            raveManager = RavePayManager.initWithCard(cardNumber: "\(cardNumber)", expiry: "\(cardExpiry)", cvv: "\(cvv)")
            raveManager.savedCardsAllow = true
            raveManager.supportedPaymentMethods = [.card]
        }
        
        raveManager.email = "\(email)"
        raveManager.delegate = self
        raveManager.transcationRef = "\(transactionRef)"
        raveManager.amount = "\(dictionary["amount"]!)"
        raveManager.country = "\(dictionary["country"]!)"
        raveManager.currencyCode = "\(currency)"
        let config = RavePayConfig.sharedConfig()
        config.publicKey = "\(publicKey)"
        config.secretKey = "\(secretKey)"
        config.isStaging = "\(isLive)" == "false"
        let controller = UIApplication.shared.delegate!.window!?.topViewController()
        raveManager.show(withController: controller!)
    }
}

extension SwiftBundleRavePayPlugin: RavePaymentManagerDelegate {
    public func ravePaymentManagerDidCancel(_ ravePaymentManager: RavePayManager) {
        bundlePayFlutterResult(FlutterError(code: "TERMINATED", message: "User cancelled", details: nil))
    }
    
    public func ravePaymentManager(_ ravePaymentManager: RavePayManager, didSucceedPaymentWithResult result: [String : AnyObject]) {
        bundlePayFlutterResult(result)
    }
    
    public func ravePaymentManager(_ ravePaymentManager: RavePayManager, didFailPaymentWithResult result: [String : AnyObject]) {
        bundlePayFlutterResult(result)
    }
    
    
}

extension UIWindow {
    func topViewController() -> UIViewController? {
        var top = self.rootViewController
        while true {
            if let presented = top?.presentingViewController {
                top = presented
            } else if let nav = top as? UINavigationController {
                top = nav.visibleViewController
            } else if let tab = top as? UITabBarController {
                top = tab.selectedViewController
            } else {
                break
            }
        }
        return top
    }
}
